import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.nio.file.FileSystems;
import java.nio.file.PathMatcher;
import java.nio.file.Paths;
import java.util.Arrays;

import org.apache.commons.compress.archivers.sevenz.SevenZArchiveEntry;
import org.apache.commons.compress.archivers.sevenz.SevenZFile;
import org.apache.commons.io.IOUtils;
import org.jetel.component.AbstractGenericTransform;
import org.jetel.data.ByteDataField;
import org.jetel.data.DataField;
import org.jetel.data.DataRecord;
import org.jetel.data.Defaults;
import org.jetel.enums.ProcessingType;
import org.jetel.util.property.RefResFlag;
import org.jetel.util.stream.StreamUtils;

/**
 * This is an example custom reader. It shows how you can
 *  create records using a data source.
 */
public class SevenZipReader extends AbstractGenericTransform {
	
	private static final Charset PASSWORD_CHARSET = Charset.forName("UTF-16LE");
	
	private String fileUrl;
	private String password;
	private String entryName;
	private ProcessingType mode;
	private PathMatcher matcher;

	private DataRecord record;
	private DataField fName;
	private ByteDataField fData;
	private DataField fEntryNumber;
	
	@Override
	public void execute() throws Exception {
		File file = getFile(fileUrl);
		
		if (!file.exists()) {
			throw new FileNotFoundException(file.getAbsolutePath());
		}
		
		byte[] bytes = (mode == ProcessingType.STREAM) ? new byte[Defaults.PortReadingWriting.DATA_LENGTH] : null;
		int entryNumber = 0;

		byte[] passwordBytes = (password != null) ? password.getBytes(PASSWORD_CHARSET) : null;
		getNode().getLog().debug(Arrays.toString(passwordBytes));
		try (SevenZFile zipFile = new SevenZFile(file, passwordBytes)) {
			SevenZArchiveEntry entry;
			while ((entry = zipFile.getNextEntry()) != null) {
				if (matcher.matches(Paths.get(entry.getName()))) {
					if (!entry.isDirectory() && entry.hasStream()) {
						fEntryNumber.setValue(entryNumber++);
						try (InputStream is = new SevenZInputStream(zipFile)) {
							fName.setValue(entry.getName());
							if (mode == ProcessingType.DISCRETE) {
								bytes = IOUtils.toByteArray(is);
								fData.setValue(bytes);
								writeRecordToPort(0, record);
							} else if (mode == ProcessingType.STREAM) {
								int count;
								while ((count = StreamUtils.readBlocking(is, bytes)) != -1) {
									fData.setValue(bytes, 0, count);
									writeRecordToPort(0, record);
								}
								fData.setNull(true);
								writeRecordToPort(0, record);
							}
							record.reset();
						}
					}
				}
			}
		}
		
	}

	@Override
	public void init() {
		super.init();

		fileUrl = getProperties().getStringProperty("fileUrl", null, RefResFlag.URL);
		password = getProperties().getStringProperty("password");
		entryName = getProperties().getStringProperty("entryName", null, RefResFlag.SPEC_CHARACTERS_OFF);
		
		mode = ProcessingType.fromString(getProperties().getStringProperty("mode"), ProcessingType.STREAM);
		
		matcher = FileSystems.getDefault().getPathMatcher("glob:" + entryName);

		record = outRecords[0];
		fName = record.getField("entryName");
		fData = (ByteDataField) record.getField("data");
		fEntryNumber = record.getField("entryNumber");
	}

	private static class SevenZInputStream extends InputStream {
		
		private final SevenZFile file;

		public SevenZInputStream(SevenZFile file) {
			this.file = file;
		}

		@Override
		public int read() throws IOException {
			return file.read();
		}

		@Override
		public int read(byte[] b, int off, int len) throws IOException {
			return file.read(b, off, len);
		}
		
	}
}
